@@ -9,9 +9,11 @@ import (
 	"strings"
 	"text/template"
 
+	"github.com/distribution/reference"
 	"github.com/spf13/cobra"
 
 	"github.com/iximiuz/cdebug/pkg/cliutil"
+	"github.com/iximiuz/cdebug/pkg/kubernetes"
 )
 
 const (
@@ -23,6 +25,37 @@ const (
 	schemaKubeLong   = "kubernetes://"
 	schemaKubeShort  = "k8s://"
 	schemaNerdctl    = "nerdctl://"
+	schemaPodman     = "podman://"
+	schemaOCI        = "oci://" // runc, crun, etc.
+
+	exampleText = `
+  # Start a %s shell in the Docker container:
+  cdebug exec -it mycontainer
+  cdebug exec -it docker://my-container
+
+  # Execute a command in the Docker container:
+  cdebug exec mycontainer cat /etc/os-release
+
+  # Use a different debugging toolkit image:
+  cdebug exec -it --image=alpine mycontainer
+
+  # Use a nixery.dev image (https://nixery.dev/):
+  cdebug exec -it --image=nixery.dev/shell/vim/ps/tshark mycontainer
+
+  # Exec into a containerd container:
+  cdebug exec -it containerd://mycontainer ...
+  cdebug exec --namespace myns -it containerd://mycontainer ...
+
+  # Exec into a nerdctl container:
+  cdebug exec -it nerdctl://mycontainer ...
+
+  # Start a shell in a Kubernetes pod:
+  cdebug exec -it pod/mypod
+  cdebug exec -it k8s://mypod
+  cdebug exec --namespace=myns -it pod/mypod
+
+  # Start a shell in a Kubernetes pod's container:
+  cdebug exec -it pod/mypod/mycontainer`
 )
 
 var (
@@ -47,6 +80,7 @@ type options struct {
 	tty        bool
 	stdin      bool
 	cmd        []string
+	user       string
 	privileged bool
 	autoRemove bool
 	quiet      bool
@@ -54,16 +88,26 @@ type options struct {
 	runtime   string
 	platform  string
 	namespace string
+
+	kubeconfig        string
+	kubeconfigContext string
+
+	override     string
+	overrideType kubernetes.OverrideType
 }
 
 func NewCommand(cli cliutil.CLI) *cobra.Command {
 	var opts options
 
 	cmd := &cobra.Command{
-		Use:   "exec [OPTIONS] CONTAINER [COMMAND] [ARG...]",
-		Short: "Start a debugger shell in the target container",
-		Args:  cobra.MinimumNArgs(1),
+		Use:     "exec [OPTIONS] [schema://][POD][CONTAINER] [COMMAND] [ARG...]",
+		Short:   "Start a debugger shell in the target container or pod.",
+		Example: fmt.Sprintf(exampleText[1:], strings.TrimPrefix(defaultToolkitImage, "docker.io/library/")),
+		Args:    cobra.MinimumNArgs(1),
 		RunE: func(cmd *cobra.Command, args []string) error {
+			if !opts.stdin {
+				opts.quiet = true
+			}
 			cli.SetQuiet(opts.quiet)
 
 			if err := cli.InputStream().CheckTty(opts.stdin, opts.tty); err != nil {
@@ -78,10 +122,23 @@ func NewCommand(cli cliutil.CLI) *cobra.Command {
 			if sep := strings.Index(opts.target, "://"); sep != -1 {
 				opts.schema = opts.target[:sep+3]
 				opts.target = opts.target[sep+3:]
+			} else if strings.HasPrefix(opts.target, "pod/") || strings.HasPrefix(opts.target, "pods/") {
+				opts.schema = schemaKubeLong
 			} else {
 				opts.schema = schemaDocker
 			}
 
+			if !reference.ReferenceRegexp.MatchString(opts.image) {
+				return cliutil.WrapStatusError(
+					fmt.Errorf("invalid debugging toolkit image name %q: %v",
+						opts.image, reference.ErrReferenceInvalidFormat),
+				)
+			}
+
+			if opts.tty && !opts.stdin {
+				return cliutil.WrapStatusError(errors.New("the -t/--tty flag requires the -i/--stdin flag"))
+			}
+
 			ctx := context.Background()
 
 			switch opts.schema {
@@ -91,8 +148,11 @@ func NewCommand(cli cliutil.CLI) *cobra.Command {
 			case schemaDocker:
 				return cliutil.WrapStatusError(runDebuggerDocker(ctx, cli, &opts))
 
-			case schemaKubeCRI, schemaKubeLong, schemaKubeShort:
-				return cliutil.WrapStatusError(errors.New("coming soon..."))
+			case schemaKubeLong, schemaKubeShort:
+				return cliutil.WrapStatusError(runDebuggerKubernetes(ctx, cli, &opts))
+
+			case schemaPodman, schemaOCI, schemaKubeCRI:
+				return cliutil.WrapStatusError(errors.New("coming soon"))
 
 			default:
 				return cliutil.WrapStatusError(fmt.Errorf("unknown schema %q", opts.schema))
@@ -136,6 +196,13 @@ func NewCommand(cli cliutil.CLI) *cobra.Command {
 		false,
 		`Allocate a pseudo-TTY (as in "docker exec -t")`,
 	)
+	flags.StringVarP(
+		&opts.user,
+		"user",
+		"u",
+		"",
+		`Run the debugger container as User (format: <name|uid>[:<group|gid>])`,
+	)
 	flags.BoolVar(
 		&opts.privileged,
 		"privileged",
@@ -167,6 +234,32 @@ func NewCommand(cli cliutil.CLI) *cobra.Command {
 		"",
 		`Platform (e.g., linux/amd64, linux/arm64) of the target container (for some runtimes it's hard to detect it automatically, but the debug sidecar must be of the same platform as the target)`,
 	)
+	flags.StringVar(
+		&opts.kubeconfig,
+		"kubeconfig",
+		"",
+		`Path to the kubeconfig file (default is $HOME/.kube/config)`,
+	)
+	flags.StringVar(
+		&opts.kubeconfigContext,
+		"kubeconfig-context",
+		"",
+		`Name of the kubeconfig context to use`,
+	)
+	flags.StringVar(
+		&opts.override,
+		"override",
+		"",
+		`[Kubernetes only] An inline JSON override for the generated ephemeral container object. Example: '{ "env": [{ "name": "DEBUG", "value": "1" }] }'`,
+	)
+	flags.StringVar(
+		(*string)(&opts.overrideType),
+		"override-type",
+		string(kubernetes.DefaultOverrideType),
+		fmt.Sprintf(`[Kubernetes only] The method used to override the generated ephemeral container object: %s, %s, or %s.`,
+			kubernetes.OverrideTypeJSON, kubernetes.OverrideTypeMerge, kubernetes.OverrideTypeStrategic,
+		),
+	)
 
 	return cmd
 }
@@ -179,21 +272,39 @@ func debuggerName(name string, runID string) string {
 }
 
 var (
+	simpleEntrypoint = template.Must(template.New("user-entrypoint").Parse(`
+set -euo pipefail
+
+if [ "${HOME:-/}" != "/" ]; then
+	ln -s /proc/{{ .TARGET_PID }}/root/ ${HOME}target-rootfs
+fi
+
+# TODO: Add target container's PATH to the user's PATH
+
+exec {{ .Cmd }}
+`))
+
 	chrootEntrypoint = template.Must(template.New("chroot-entrypoint").Parse(`
 set -euo pipefail
 
+CURRENT_PID=$(sh -c 'echo $PPID')
+
 {{ if .IsNix }}
-rm -rf /proc/{{ .PID }}/root/nix
-ln -s /proc/$$/root/nix /proc/{{ .PID }}/root/nix
+CURRENT_NIX_INODE=$(stat -c '%i' /nix)
+TARGET_NIX_INODE=$(stat -c '%i' /proc/{{ .TARGET_PID }}/root/nix 2>/dev/null || echo 0)
+if [ ${CURRENT_NIX_INODE} -ne ${TARGET_NIX_INODE} ]; then
+  rm -rf /proc/{{ .TARGET_PID }}/root/nix
+  ln -s /proc/${CURRENT_PID}/root/nix /proc/{{ .TARGET_PID }}/root/nix
+fi
 {{ end }}
 
-ln -s /proc/$$/root/bin/ /proc/{{ .PID }}/root/.cdebug-{{ .ID }}
+ln -s /proc/${CURRENT_PID}/root/bin/ /proc/{{ .TARGET_PID }}/root/.cdebug-{{ .ID }}
 
 cat > /.cdebug-entrypoint.sh <<EOF
 #!/bin/sh
 export PATH=$PATH:/.cdebug-{{ .ID }}
 
-chroot /proc/{{ .PID }}/root {{ .Cmd }}
+chroot /proc/{{ .TARGET_PID }}/root {{ .Cmd }}
 EOF
 
 exec sh /.cdebug-entrypoint.sh
@@ -206,26 +317,36 @@ func debuggerEntrypoint(
 	targetPID int,
 	image string,
 	cmd []string,
+	chroot bool,
 ) string {
+	if chroot {
+		return mustRenderTemplate(
+			cli,
+			chrootEntrypoint,
+			map[string]any{
+				"ID":         runID,
+				"TARGET_PID": targetPID,
+				"IsNix":      strings.Contains(image, "nixery"),
+				"Cmd": func() string {
+					if len(cmd) == 0 {
+						return ""
+					}
+					return "sh -c '" + strings.Join(shellescape(cmd), " ") + "'"
+				}(),
+			},
+		)
+	}
+
 	return mustRenderTemplate(
 		cli,
-		chrootEntrypoint,
+		simpleEntrypoint,
 		map[string]any{
-			"ID":    runID,
-			"PID":   targetPID,
-			"IsNix": strings.Contains(image, "nixery"),
+			"PID": targetPID,
 			"Cmd": func() string {
 				if len(cmd) == 0 {
-					// bash provides a much better UX out of the box, so
-					// let's try to use bash if we know it's likely available.
-					if strings.HasPrefix(image, "nixery.dev") && strings.Contains(image, "/shell") {
-						return "bash"
-					}
-
-					// Default to sh otherwise.
 					return "sh"
 				}
-				return "sh -c '" + strings.Join(shellescape(cmd), " ") + "'"
+				return strings.Join(shellescape(cmd), " ")
 			}(),
 		},
 	)
@@ -250,3 +371,7 @@ func shellescape(args []string) (escaped []string) {
 	}
 	return
 }
+
+func isRootUser(user string) bool {
+	return len(user) == 0 || user == "root" || user == "0" || user == "0:0"
+}
