@@ -112,7 +112,7 @@ func runDebuggerContainerd(ctx context.Context, cli cliutil.CLI, opts *options)
 				oci.WithDefaultPathEnv,
 				oci.WithImageConfig(image), // May override the default $PATH.
 				oci.WithProcessArgs("sh", "-c", debuggerEntrypoint(
-					cli, runID, targetPID, opts.image, opts.cmd,
+					cli, runID, targetPID, opts.image, opts.cmd, isRootUser(opts.user),
 				)),
 				func() oci.SpecOpts {
 					if opts.tty {
@@ -120,6 +120,12 @@ func runDebuggerContainerd(ctx context.Context, cli cliutil.CLI, opts *options)
 					}
 					return ociSpecNoOp
 				}(),
+				func() oci.SpecOpts {
+					if opts.user != "" {
+						return oci.WithUser(opts.user)
+					}
+					return ociSpecNoOp
+				}(),
 				func() oci.SpecOpts {
 					if opts.privileged {
 						return oci.WithPrivileged
